# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: jupyrest.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


DESCRIPTOR = _descriptor.FileDescriptor(
    name="jupyrest.proto",
    package="",
    syntax="proto3",
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
    serialized_pb=b'\n\x0ejupyrest.proto"4\n\x06Status\x12\x19\n\x04\x63ode\x18\x01 \x01(\x0e\x32\x0b.StatusCode\x12\x0f\n\x07\x64\x65tails\x18\x02 \x01(\t"\x19\n\x08Notebook\x12\r\n\x05ipynb\x18\x01 \x01(\x0c"s\n\x18NotebookExecutionRequest\x12\x13\n\x0bnotebook_id\x18\x01 \x01(\t\x12\x13\n\x0bplugin_name\x18\x02 \x01(\t\x12\x12\n\nparameters\x18\x03 \x01(\x0c\x12\x19\n\x11parameterize_only\x18\x04 \x01(\x08"Q\n\x19NotebookExecutionResponse\x12\x17\n\x06status\x18\x01 \x01(\x0b\x32\x07.Status\x12\x1b\n\x08notebook\x18\x02 \x01(\x0b\x32\t.Notebook*\x88\x01\n\nStatusCode\x12\x06\n\x02OK\x10\x00\x12\x12\n\x0eINTERNAL_ERROR\x10\x01\x12\r\n\tBAD_INPUT\x10\x02\x12!\n\x1dINPUT_SCHEMA_VALIDATION_ERROR\x10\x03\x12\x16\n\x12NOTEBOOK_NOT_FOUND\x10\x04\x12\x14\n\x10PLUGIN_NOT_FOUND\x10\x05\x32Z\n\rWorkerService\x12I\n\x10\x65xecute_notebook\x12\x19.NotebookExecutionRequest\x1a\x1a.NotebookExecutionResponseb\x06proto3',
)

_STATUSCODE = _descriptor.EnumDescriptor(
    name="StatusCode",
    full_name="StatusCode",
    filename=None,
    file=DESCRIPTOR,
    create_key=_descriptor._internal_create_key,
    values=[
        _descriptor.EnumValueDescriptor(
            name="OK",
            index=0,
            number=0,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.EnumValueDescriptor(
            name="INTERNAL_ERROR",
            index=1,
            number=1,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.EnumValueDescriptor(
            name="BAD_INPUT",
            index=2,
            number=2,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.EnumValueDescriptor(
            name="INPUT_SCHEMA_VALIDATION_ERROR",
            index=3,
            number=3,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.EnumValueDescriptor(
            name="NOTEBOOK_NOT_FOUND",
            index=4,
            number=4,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.EnumValueDescriptor(
            name="PLUGIN_NOT_FOUND",
            index=5,
            number=5,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    containing_type=None,
    serialized_options=None,
    serialized_start=300,
    serialized_end=436,
)
_sym_db.RegisterEnumDescriptor(_STATUSCODE)

StatusCode = enum_type_wrapper.EnumTypeWrapper(_STATUSCODE)
OK = 0
INTERNAL_ERROR = 1
BAD_INPUT = 2
INPUT_SCHEMA_VALIDATION_ERROR = 3
NOTEBOOK_NOT_FOUND = 4
PLUGIN_NOT_FOUND = 5


_STATUS = _descriptor.Descriptor(
    name="Status",
    full_name="Status",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="code",
            full_name="Status.code",
            index=0,
            number=1,
            type=14,
            cpp_type=8,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="details",
            full_name="Status.details",
            index=1,
            number=2,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=18,
    serialized_end=70,
)


_NOTEBOOK = _descriptor.Descriptor(
    name="Notebook",
    full_name="Notebook",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="ipynb",
            full_name="Notebook.ipynb",
            index=0,
            number=1,
            type=12,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"",
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=72,
    serialized_end=97,
)


_NOTEBOOKEXECUTIONREQUEST = _descriptor.Descriptor(
    name="NotebookExecutionRequest",
    full_name="NotebookExecutionRequest",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="notebook_id",
            full_name="NotebookExecutionRequest.notebook_id",
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="plugin_name",
            full_name="NotebookExecutionRequest.plugin_name",
            index=1,
            number=2,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="parameters",
            full_name="NotebookExecutionRequest.parameters",
            index=2,
            number=3,
            type=12,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"",
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="parameterize_only",
            full_name="NotebookExecutionRequest.parameterize_only",
            index=3,
            number=4,
            type=8,
            cpp_type=7,
            label=1,
            has_default_value=False,
            default_value=False,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=99,
    serialized_end=214,
)


_NOTEBOOKEXECUTIONRESPONSE = _descriptor.Descriptor(
    name="NotebookExecutionResponse",
    full_name="NotebookExecutionResponse",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="status",
            full_name="NotebookExecutionResponse.status",
            index=0,
            number=1,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="notebook",
            full_name="NotebookExecutionResponse.notebook",
            index=1,
            number=2,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=216,
    serialized_end=297,
)

_STATUS.fields_by_name["code"].enum_type = _STATUSCODE
_NOTEBOOKEXECUTIONRESPONSE.fields_by_name["status"].message_type = _STATUS
_NOTEBOOKEXECUTIONRESPONSE.fields_by_name["notebook"].message_type = _NOTEBOOK
DESCRIPTOR.message_types_by_name["Status"] = _STATUS
DESCRIPTOR.message_types_by_name["Notebook"] = _NOTEBOOK
DESCRIPTOR.message_types_by_name["NotebookExecutionRequest"] = _NOTEBOOKEXECUTIONREQUEST
DESCRIPTOR.message_types_by_name[
    "NotebookExecutionResponse"
] = _NOTEBOOKEXECUTIONRESPONSE
DESCRIPTOR.enum_types_by_name["StatusCode"] = _STATUSCODE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Status = _reflection.GeneratedProtocolMessageType(
    "Status",
    (_message.Message,),
    {
        "DESCRIPTOR": _STATUS,
        "__module__": "jupyrest_pb2"
        # @@protoc_insertion_point(class_scope:Status)
    },
)
_sym_db.RegisterMessage(Status)

Notebook = _reflection.GeneratedProtocolMessageType(
    "Notebook",
    (_message.Message,),
    {
        "DESCRIPTOR": _NOTEBOOK,
        "__module__": "jupyrest_pb2"
        # @@protoc_insertion_point(class_scope:Notebook)
    },
)
_sym_db.RegisterMessage(Notebook)

NotebookExecutionRequest = _reflection.GeneratedProtocolMessageType(
    "NotebookExecutionRequest",
    (_message.Message,),
    {
        "DESCRIPTOR": _NOTEBOOKEXECUTIONREQUEST,
        "__module__": "jupyrest_pb2"
        # @@protoc_insertion_point(class_scope:NotebookExecutionRequest)
    },
)
_sym_db.RegisterMessage(NotebookExecutionRequest)

NotebookExecutionResponse = _reflection.GeneratedProtocolMessageType(
    "NotebookExecutionResponse",
    (_message.Message,),
    {
        "DESCRIPTOR": _NOTEBOOKEXECUTIONRESPONSE,
        "__module__": "jupyrest_pb2"
        # @@protoc_insertion_point(class_scope:NotebookExecutionResponse)
    },
)
_sym_db.RegisterMessage(NotebookExecutionResponse)


_WORKERSERVICE = _descriptor.ServiceDescriptor(
    name="WorkerService",
    full_name="WorkerService",
    file=DESCRIPTOR,
    index=0,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
    serialized_start=438,
    serialized_end=528,
    methods=[
        _descriptor.MethodDescriptor(
            name="execute_notebook",
            full_name="WorkerService.execute_notebook",
            index=0,
            containing_service=None,
            input_type=_NOTEBOOKEXECUTIONREQUEST,
            output_type=_NOTEBOOKEXECUTIONRESPONSE,
            serialized_options=None,
            create_key=_descriptor._internal_create_key,
        ),
    ],
)
_sym_db.RegisterServiceDescriptor(_WORKERSERVICE)

DESCRIPTOR.services_by_name["WorkerService"] = _WORKERSERVICE

# @@protoc_insertion_point(module_scope)
