"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import google.protobuf.descriptor
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message
import typing
import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor = ...

class StatusCode(_StatusCode, metaclass=_StatusCodeEnumTypeWrapper):
    pass
class _StatusCode:
    V = typing.NewType('V', builtins.int)
class _StatusCodeEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_StatusCode.V], builtins.type):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor = ...
    OK = StatusCode.V(0)
    INTERNAL_ERROR = StatusCode.V(1)
    BAD_INPUT = StatusCode.V(2)
    INPUT_SCHEMA_VALIDATION_ERROR = StatusCode.V(3)
    NOTEBOOK_NOT_FOUND = StatusCode.V(4)
    PLUGIN_NOT_FOUND = StatusCode.V(5)

OK = StatusCode.V(0)
INTERNAL_ERROR = StatusCode.V(1)
BAD_INPUT = StatusCode.V(2)
INPUT_SCHEMA_VALIDATION_ERROR = StatusCode.V(3)
NOTEBOOK_NOT_FOUND = StatusCode.V(4)
PLUGIN_NOT_FOUND = StatusCode.V(5)
global___StatusCode = StatusCode


class Status(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    CODE_FIELD_NUMBER: builtins.int
    DETAILS_FIELD_NUMBER: builtins.int
    code: global___StatusCode.V = ...
    details: typing.Text = ...
    def __init__(self,
        *,
        code : global___StatusCode.V = ...,
        details : typing.Text = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"code",b"code",u"details",b"details"]) -> None: ...
global___Status = Status

class Notebook(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    IPYNB_FIELD_NUMBER: builtins.int
    ipynb: builtins.bytes = ...
    def __init__(self,
        *,
        ipynb : builtins.bytes = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"ipynb",b"ipynb"]) -> None: ...
global___Notebook = Notebook

class NotebookExecutionRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    NOTEBOOK_ID_FIELD_NUMBER: builtins.int
    PLUGIN_NAME_FIELD_NUMBER: builtins.int
    PARAMETERS_FIELD_NUMBER: builtins.int
    PARAMETERIZE_ONLY_FIELD_NUMBER: builtins.int
    notebook_id: typing.Text = ...
    plugin_name: typing.Text = ...
    parameters: builtins.bytes = ...
    parameterize_only: builtins.bool = ...
    def __init__(self,
        *,
        notebook_id : typing.Text = ...,
        plugin_name : typing.Text = ...,
        parameters : builtins.bytes = ...,
        parameterize_only : builtins.bool = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"notebook_id",b"notebook_id",u"parameterize_only",b"parameterize_only",u"parameters",b"parameters",u"plugin_name",b"plugin_name"]) -> None: ...
global___NotebookExecutionRequest = NotebookExecutionRequest

class NotebookExecutionResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    STATUS_FIELD_NUMBER: builtins.int
    NOTEBOOK_FIELD_NUMBER: builtins.int
    @property
    def status(self) -> global___Status: ...
    @property
    def notebook(self) -> global___Notebook: ...
    def __init__(self,
        *,
        status : typing.Optional[global___Status] = ...,
        notebook : typing.Optional[global___Notebook] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal[u"notebook",b"notebook",u"status",b"status"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"notebook",b"notebook",u"status",b"status"]) -> None: ...
global___NotebookExecutionResponse = NotebookExecutionResponse
